# St terminal for Fatdog64

This package leverages st-flexipatch, a custom st build that integrates several
community patches enhancing st. Patch source code is only compiled if patches.h
defines the symbol identifying a specific patch. Patches.h is initialized from
patches.def.h, then edited forever to select one's preferred patchset. The
provided patches.h enables the 32 (of 70) specific patches I use.

Configuration data is defined in config.h -- created and customized similarly
to patches.h. The provided config.h contains the generic configuration for
st, mixed with the configuration specific to my patchset and my key binding
preferences.

Normally, a patch is enabled by defining its cpp symbol equal to 1, and disabled
with 0. However, I have added three #defines, FOR_TMUX, FOR_BOTH, FOR_NONE,
meaning:

```
            THE PATCH IS...
            ---------------------------------------------------
 FOR_TMUX   included in an st build optimized for use with tmux
!FOR_TMUX   excluded from use with tmux
 FOR_BOTH   included in any build
 FOR_NONE   excluded from all builds
```

Hence one can build two st binary flavors:

1. optimized for use with tmux by defining `FOR_TMUX=1`;
2. for generic use, by defining `FOR_TMUX=0`.

The provided pkgbuild recipe builds the latter.
The two binaries differ only slightly:

```
FEATURE                 GENERIC   TMUX
-------                 -------   --------
scrollback buffer size  20,000    2,000
COLUMNS_PATCH           included  excluded
OPENURLONCLICK_PATCH    included  excluded
```

The two patches are only included with the generic flavor because tmux already
provides equivalent features. All other patches appear equally in both flavors.

## Description

Both binary flavors can do lots, e.g., display SIXEL images, resize the terminal
font, font ligatures, color configuration via XResources, drag-and-drop target,
click URL, and more.  Refer to patches.h for a full list. To customize key
bindings refer to the FIXKEYBOARDINPUT_PATCH section in config.h.

Notable key bindings:

```
copy/paste clipboard        |  S-C-C      | S-C-V
font size                   |  S-C-PgUp   | S-C-PgDn / S-C-Home
invert terminal colors      |  S-C-X      |
new st terminal             |  S-C-Enter  |
open URL                    |  click      |
select(³)                   |  mouse⁽²⁾   | [2click](fragment), [3click](line)
this menu with selection⁽¹⁾ |  A-M        | [rclick]⁽²⁾
this menu with term buf⁽¹⁾  |  S-C-M      | [rclick]⁽²⁾
```
⁽²⁾ hold S for tmux, works with [2c,3c] too  
(³) no mouse? Use S-C-Esc  

## Bugs

Annoyance - The st terminal prints a error message when it receives unsupported
terminal control codes. Most other terminals keep quiet about foreign codes.

Occasionally, when I hit a quick series of backspaces, I can see extraneous
letters creeping in. This doesn't bother me because halting and backspacing
again usually clears the boo-boo. I suspect this is due to my custom key
bindings in config.h, or perhaps to some timing issue involving tmux.

## Future

The provided config.h file mentions st-plumber, a shell script I wrote to add
even more features, e.g. a theme switcher, tmux buffer saving, and more. Because
the script is a work-in-progress, I decided not to add it to the package for
now. To be honest, I rarely use st-plumber because the patched st + tmux cover
all my needs, and I set terminal colors once and forever with XResources.

_step 2025-04-05_

